#!/usr/bin/env ruby
#encoding: utf-8

module MinifyResources

  PARENT_DIR    = 'public/';
  MANIFEST_NAME = '/manifest.txt'

  VERSION_FILE    = 'blobs_versions.txt'
  VERSION_FORMAT  = '?v=#'
  INIT_VERSION    = 1

  CSS_ORIGINAL_DIRS = ['css']
  CSS_MINIFIED_DIR  = ['css']
  CSS_BLOB          = 'css/blob.css'
  CSS_DIR_TO_MINIFY = 'css'
  CSS_TYPE          = 'css'

  JS_ORIGINAL_DIRS  = ['js/third_party', 'js/gazelle']
  JS_MINIFIED_DIR   = ['js/minified']
  JS_BLOB           = 'js/minified/blob.js'
  JS_DIR_TO_MINIFY  = 'js/gazelle'
  JS_TYPE           = 'js'

  def self.get_original_css_files
    css_files = []
    CSS_ORIGINAL_DIRS.each { |dir| css_files.concat(get_files_from_dir(dir, false)) }
    css_files
  end

  def self.get_original_js_files
    js_files = []
    JS_ORIGINAL_DIRS.each { |dir| js_files.concat(get_files_from_dir(dir, false)) }
    js_files
  end

  def self.get_minified_js_files
    js_files = []
    blob_version = get_blob_version(JS_TYPE)
    JS_MINIFIED_DIR.each do |dir|
      files_in_dir = get_files_from_dir(dir, false)
      index_of_blob = files_in_dir.index(JS_BLOB)
      files_in_dir[index_of_blob] += VERSION_FORMAT.sub("#", blob_version.to_s) if index_of_blob
      js_files.concat(files_in_dir)
    end
    js_files
  end

  def self.get_minified_css_files
    css_files = []
    blob_version = get_blob_version(CSS_TYPE)
    css_files << CSS_BLOB + VERSION_FORMAT.sub("#", blob_version.to_s)
    css_files
  end

  def self.minify_all
    require 'jsmin'
    require 'cssmin'
    minify(::JSMin, JS_DIR_TO_MINIFY, PARENT_DIR + JS_BLOB, JS_TYPE)
    minify(::CSSMin, CSS_DIR_TO_MINIFY, PARENT_DIR + CSS_BLOB, CSS_TYPE)
  end

  def self.minify(lib, source_dir, output, type)
    files = get_files_from_dir(source_dir, true)
    return if files.empty?
    require_relative 'helpers/nice_bytes'
    raw_size = 0
    minified = files.map do |file|
      raw = IO.read(file)
      raw_size += raw.length
      lib.minify(raw.dup)
    end.join
    min_size = minified.length
    existing = File.exists?(output) && IO.read(output)
    if minified == existing
      puts "no change to #{output}"
    else
      File.open(output, 'w') { |f| f << minified }
      increment_blob_version(type)
      puts "%i files => %s; %s => %s (%.1f%% reduction)" % [
        files.length,
        output,
        NiceBytes.nice_bytes(raw_size),
        NiceBytes.nice_bytes(min_size),
        100 * (raw_size-min_size) / raw_size
      ]
    end
  end

  private
  # Returns files from specified directory
  # if withParentDir is true than files are returned with parent directory prefix ('public/')
  def self.get_files_from_dir(dir, withParentDir)
    files = []
    IO.read(PARENT_DIR + dir + MANIFEST_NAME).scan(/\S+/).each { |file| files << (withParentDir ? PARENT_DIR : '') + dir + '/' + file }
    files
  end

  def self.get_blobs_versions()
    versions = YAML.load(File.read(VERSION_FILE)) if File.exists?(VERSION_FILE)
    versions && versions.is_a?(Hash) ? versions : { JS_TYPE => INIT_VERSION, CSS_TYPE => INIT_VERSION }
  end

  def self.get_blob_version(type)
    versions = get_blobs_versions
    versions[type] != nil ? versions[type].to_i : INIT_VERSION
  end

  def self.increment_blob_version(type)
    versions = get_blobs_versions
    versions[type] = versions[type] != nil ? versions[type].to_i + 1 : INIT_VERSION
    File.open(VERSION_FILE, 'w') { |f| f.write(YAML.dump(versions)) }
  end

end

if __FILE__ == $0
  require 'yaml'
  MinifyResources.minify_all
end

# vim:ff=unix ts=2 ss=2
